@@ -6,11 +6,12 @@ OUT = (("-o","--output"),{'type':str,'dest':'out_file','help':"write path to OUT
 INDIRECT = (("-i","--indirect"),{'action':'store_true','dest':'indirect','default':False,'help':"transform indirectly (default: False)"})
 PENALIZE = (("-p","--penalize"),{'type':int,'default':20,'dest':'penalize','help':"penalize air time by factor PENALTY",'metavar':'PENALTY'})
 VISUALIZE = (("-m","--map"),{'action':'store_true','dest':'visualize','default':False,'help':"render route to HTML map"})
+BENCHMARK = (("-b","--benchmark"),{'type':str,'dest':'benchmark','help':"specify file with routes to benchmark",'metavar':'ROUTES'})
 
 CONFIG = [
     ("npz",{'help':"route using NPZ graph",'args':[INDIRECT,PENALIZE,SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
-    ("nx",{'help':"route using NX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
-    ("cnx",{'help':"route using CNX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
+    ("nx",{'help':"route using NX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,BENCHMARK,GRAPH]}),
+    ("cnx",{'help':"route using CNX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,BENCHMARK,GRAPH]}),
     ("gt",{'help':"route using GT graph",'args':[INDIRECT,PENALIZE,SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
     ("direct",{'help':"route directly",'args':[
         (("-s","--source"),{'type':int,'dest':'source_id','help':"set source power tower to SOURCE (default: random)",'metavar':'SOURCE','required':True}),
@@ -164,13 +165,17 @@ def astar_nx(g,source_id=None,target_id=None):
         path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
     return path
 
-def route_nx(file_name,source_id=None,target_id=None,out_file=None,visualize=False):
+def route_nx(file_name,source_id=None,target_id=None,out_file=None,visualize=False,benchmark=None):
     from limic.util import start, end, load_pickled, save_path
     start("Loading from",file_name)
     g = load_pickled(file_name)
     end()
     start("Routing using NX")
-    path = astar_nx(g,source_id,target_id)
+    if benchmark:
+        for source, target in load_pickled(benchmark):
+            path = astar_nx(g,(source,),(target,))
+    else:
+        path = astar_nx(g,source_id,target_id)
     end()
     start("Saving path to",out_file)
     save_path(path,out_file,visualize)
@@ -192,7 +197,11 @@ def astar_cnx(g,source_id=None,target_id=None):
             cost = dist = distance(source,target)
             path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
             return path
-        path = astar_nx(c,(source,source_id),(target,target_id))
+        path = astar_nx(node2c[source],(source,),(target,))
+        return path
+    if target not in node2closest:
+        cost = dist = distance(source,target)
+        path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
         return path
     paths = []
     def extend(cp1,cp2):
@@ -209,26 +218,31 @@ def astar_cnx(g,source_id=None,target_id=None):
                 cost = dist = distance(source,target)
                 path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
                 return path
-            cp = astar_nx(ps,(source,None),(ns,None))
-            cpc = astar_nx(c,(ns,None),(nt,None))
+            cp = astar_nx(ps,(source,),(ns,))
+            cpc = astar_nx(c,(ns,),(nt,))
             for u,v in zip(cpc[1],cpc[1][1:]):
                 u,v = u[3:], v[3:]
                 cuv = c[u][v]
                 sg = cuv["path"]
-                cpuv = astar_nx(sg,(u,None),(v,None))
+                cpuv = astar_nx(sg,(u,),(v,))
                 cp = extend(cp,cpuv)
-            cpt = astar_nx(pt,(nt,None),(target,None))
+            cpt = astar_nx(pt,(nt,),(target,))
             cp = extend(cp,cpt)
             paths.append(cp)
+#            paths.append((cs+c.graph['lengths'][ns][nt]+ct,[]))
     return min(paths)
 
-def route_cnx(file_name,source_id=None,target_id=None,out_file=None,visualize=False):
+def route_cnx(file_name,source_id=None,target_id=None,out_file=None,visualize=False,benchmark=None):
     from limic.util import start, end, load_pickled, save_path
     start("Loading from",file_name)
     g = load_pickled(file_name)
     end()
     start("Routing using condensed NX")
-    path = astar_cnx(g,source_id,target_id)
+    if benchmark:
+        for source, target in load_pickled(benchmark):
+            path = astar_cnx(g,(source,),(target,))
+    else:
+        path = astar_cnx(g,source_id,target_id)
     end()
     start("Saving path to",out_file)
     save_path(path,out_file,visualize)
